/************************************************
 * General-purpose Utilities for script modules
 */

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef _SCRIPTLIB_H
#define _SCRIPTLIB_H

#include "lg/objstd.h"
#include "lg/interface.h"
#include "lg/script.h"
#include "lg/scrservices.h"
#include "lg/scrmanagers.h"
#include "lg/links.h"


/****************
 * Globals
 * The hosting app/module must assign these.
 */

extern IMalloc* g_pMalloc;
extern IScriptMan* g_pScriptManager;

#ifdef _HACKS
extern bool g_fIsEditor;
extern char* g_pszFileName;
#endif

extern const cMultiParm g_mpUndef;

/*************
 * Properties
 */

// The object param is stored in the Editor\DesignNote property.
// Strings are allocated using g_pMalloc. Please use Free.

// Returns the whole Design Note.
// Useful to save time if you're reading a lot of params.
char* GetObjectParams(int iObject);
int SetObjectParams(int iObject, const char* pszParams);
// Returns the param as a particular type.
char* GetObjectParamString(int iObject, const char* pszParam);
int GetObjectParamInt(int iObject, const char* pszParam);
float GetObjectParamFloat(int iObject, const char* pszParam);
// Same, but uses the supplied string instead of extracting from the object.
char* GetParamString(const char* pszString, const char* pszParam);
int GetParamInt(const char* pszString, const char* pszParam);
float GetParamFloat(const char* pszString, const char* pszParam);

int RemoveObjectParam(int iObject, const char* pszParam);


/***********
 * Metaproperties
 */

int AddSingleMetaProperty(const char* pszMetaProp, int iObjId);
int AddSingleMetaProperty(int iMetaProp, int iObjId);
int RemoveSingleMetaProperty(const char* pszMetaProp, int iObjId);
int RemoveSingleMetaProperty(int iMetaProp, int iObjId);


/********
 * Links
 */
typedef int (*LinkIterCallbackFunc)(ILinkSrv*,ILinkQuery*,IScript*,void*);

int GetOneLinkDest(const char* pszFlavor, int iSource, int* piRealSource);
long FindLinkByData(const char* pszFlavor, int iSource, int iDest, sLink* pResult, const void* pvData, int iDataLen);
long IterateLinks(const char* pszFlavor, int iSource, int iDest,
			    LinkIterCallbackFunc pfnLinkIterCallback, IScript* pScript, void* pData);
long IterateLinksByData(const char* pszFlavor, int iSource, int iDest,
			    const void* pvFilter, int iFilterLen,
			    LinkIterCallbackFunc pfnLinkIterCallback, IScript* pScript, void* pData);
long GetAnyLink(const char* pszFlavor, int iSource, int iDest, sLink* pResult);
long GetAnyLinkByData(const char* pszFlavor, int iSource, int iDest, sLink* pResult,
			    const void* pvData, int iDataLen);
void FixupPlayerLinks(int iSource, int iPlayer);


/***********
 * Messages
 */

void SimpleSend(int iSource, int iDest, const char* pszMessage,
		    const cMultiParm& mpData1 = g_mpUndef, const cMultiParm& mpData2 = g_mpUndef, const cMultiParm& mpData3 = g_mpUndef);
void SimplePost(int iSource, int iDest, const char* pszMessage,
		    const cMultiParm& mpData1 = g_mpUndef, const cMultiParm& mpData2 = g_mpUndef, const cMultiParm& mpData3 = g_mpUndef);
long CDSend(const char* pszMessage, int iSource, const cMultiParm& mpData = g_mpUndef);


/*************
 * MultiParms
 */

void MultiParmAssignInt(sMultiParm* pParm, int iValue);
void MultiParmAssignFloat(sMultiParm* pParm, float fValue);
void MultiParmAssignString(sMultiParm* pParm, const char* pszValue);
void MultiParmAssignVector(sMultiParm* pParm, const mxs_vector* pvValue);


/***********
 * User
 */

int ShowBook(int iObject, bool bArt, int iTime = 5000);
void ShowString(const char* pszStr, int iTime, COLORREF color = 0);
__attribute__((format (printf,1,2))) // GCC extension. You have been warned.
	void DebugPrintf(const char* pszFormat, ...);
__attribute__((format (printf,1,2)))
	void DisplayPrintf(const char* pszFormat, ...);


/************
 * Utils
 */
__attribute__((nonnull (1,2)))
	char* strsep(char** str, const char* delim);
__attribute__((nonnull (1,2,3)))
	char* strqsep(char** str, const char* delim, const char* quotes);

#endif // _SCRIPTLIB_H
