
#include "lg/interface.h"
#include "lg/scrservices.h"
#include "ScriptLib.h"
#include "GenericScript.h"

#if (_DARKGAME == 3)
#define TIMING_PROPNAME	"DelayTime"
#define TRAPFLAGS_PROPNAME	"TripFlags"
#else
#define TIMING_PROPNAME	"ScriptTiming"
#define TRAPFLAGS_PROPNAME	"TrapFlags"
#endif

void cScrGeneric::InitTrapVars()
{
#if (_DARKGAME != 1)
	SService<IPropertySrv> pPropSrv(g_pScriptManager);
	if (pPropSrv->Possessed(m_iObjId, TIMING_PROPNAME))
	{
		cMultiParm mpTiming = 0;
		pPropSrv->Get(mpTiming, m_iObjId, TIMING_PROPNAME, NULL);
#if (_DARKGAME == 3)
		m_iTiming = int(static_cast<float>(mpTiming) * 1000.0);
#else
		m_iTiming = static_cast<int>(mpTiming);
#endif
	}
	if (pPropSrv->Possessed(m_iObjId, TRAPFLAGS_PROPNAME))
	{
		cMultiParm mpTrapFlags = 0;
		pPropSrv->Get(mpTrapFlags, m_iObjId, TRAPFLAGS_PROPNAME, NULL);
		m_iFlags = static_cast<int>(mpTrapFlags);
	}
#if (_DARKGAME == 3)
	// Translate TripFlags to TrapFlags
	if (m_iFlags != 0)
	{
		int iFlags = (m_iFlags & 0x18) >> 3;
		iFlags |= (m_iFlags & 0x2) << 1;
		iFlags |= (m_iFlags & 0x1) << 3;
		m_iFlags = iFlags;
	}
#endif
#endif
}

bool cScrGeneric::IsLocked(sScrMsg* pMsg)
{
	SService<ILockSrv> pLock(g_pScriptManager);
	return pLock->IsLocked(m_iObjId);
}

void cScrGeneric::SetLock(bool bLock)
{
	SService<IPropertySrv> pPropSrv(g_pScriptManager);
	cMultiParm mpLock = static_cast<int>(bLock);
	pPropSrv->Set(m_iObjId, "Locked", mpLock);
}

long __stdcall cScrGeneric::GenericTimer(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	sScrTimerMsg* pTimerMsg = static_cast<sScrTimerMsg*>(pMsg);
	const char* pszMsgName = static_cast<const char*>(pTimerMsg->name);
	m_iTiming = 0;
	m_iFlags = 0;
	if (!stricmp(pszMsgName, "TurnOn"))
	{
		return OnTurnOn(pMsg, pReply);
	}
	else if (!stricmp(pszMsgName, "TurnOff"))
	{
		return OnTurnOff(pMsg, pReply);
	}
	return 1;
}

long __stdcall cScrGeneric::GenericTurnOn(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	InitTrapVars();
	if (IsLocked(pMsg))
	{
		return 1;
	}
	if (m_iFlags & kTrapFlagNoOn)
		return 1;
	long iRet;
	const char* pszInverseMsg;
	if (m_iFlags & kTrapFlagInvert)
	{
		iRet = OnTurnOff(pMsg, pReply);
		pszInverseMsg = "TurnOn";
	}
	else
	{
		iRet = OnTurnOn(pMsg, pReply);
		pszInverseMsg = "TurnOff";
	}
	if (iRet == 0)
	{
		if (m_iTiming > 0)
			g_pScriptManager->SetTimedMessage2(m_iObjId, pszInverseMsg, m_iTiming, kSTM_OneShot, 0);
		if (m_iFlags & kTrapFlagOnce)
		{
			SetLock(true);
		}
	}
	return iRet;
}

long __stdcall cScrGeneric::GenericTurnOff(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	InitTrapVars();
	if (IsLocked(pMsg))
	{
		return 1;
	}
	if (m_iFlags & kTrapFlagNoOff)
		return 1;
	long iRet;
	//const char* pszInverseMsg;
	if (m_iFlags & kTrapFlagInvert)
	{
		iRet = OnTurnOn(pMsg, pReply);
		//pszInverseMsg = "TurnOff";
	}
	else
	{
		iRet = OnTurnOff(pMsg, pReply);
		//pszInverseMsg = "TurnOn";
	}
	if (iRet == 0)
	{
		if (m_iFlags & kTrapFlagOnce)
		{
			SetLock(true);
		}
	}
	return iRet;
}

long __stdcall cScrGeneric::GenericScriptControl(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	InitTrapVars();
	if (IsLocked(pMsg))
	{
		return 1;
	}
	long iRet = OnControl(pMsg, pReply);
	return iRet;
}

long __stdcall cScrGeneric::DelayedInit(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	sScrTimerMsg* pTimerMsg = static_cast<sScrTimerMsg*>(pMsg);
	if (stricmp(static_cast<const char*>(pTimerMsg->name), "DelayedInit"))
		return 1;
	if (pTimerMsg->data.type == kMT_String && 
		!stricmp(pTimerMsg->data.psz, "FixupPlayerLinks"))
	{
		SService<IObjectSrv> pOS(g_pScriptManager);
		object oPlayer;
		pOS->Named(oPlayer, "Player");
		if (oPlayer != 0)
			::FixupPlayerLinks(m_iObjId, oPlayer);
		UnregisterMessageHandler("Timer", static_cast<MessageHandlerFunc>(&cScrGeneric::DelayedInit));
	}
	return 0;
}

int cScrGeneric::FixupPlayerLinks(void)
{
	SService<IObjectSrv> pOS(g_pScriptManager);
	object oPlayer;
	pOS->Named(oPlayer, "Player");
	if (oPlayer != 0)
	{
		::FixupPlayerLinks(m_iObjId, oPlayer);
		return 0;
	}
	else
	{
		if (RegisterMessageHandler("Timer", static_cast<MessageHandlerFunc>(&cScrGeneric::DelayedInit)))
			g_pScriptManager->SetTimedMessage2(m_iObjId, "DelayedInit", 1, kSTM_OneShot, "FixupPlayerLinks");
		return 1;
	}
}

