
#include <windows.h>
#include <objbase.h>

#include "lg/malloc.h"
#include "lg/scrmanagers.h"
#include "lg/scrservices.h"

#include "ScriptLib.h"
#include "ScriptModule.h"

typedef int (__cdecl *MPrintfProc)(const char*, ...);

cScriptModule::~cScriptModule()
{
}

cScriptModule::cScriptModule()
	    : m_iRef(1), m_pszName(NULL), m_iCurIndex(0)
{
}

cScriptModule::cScriptModule(char* pszName)
	    : m_iRef(1), m_pszName(pszName), m_iCurIndex(0)
{
}

HRESULT __stdcall cScriptModule::QueryInterface(REFIID riid, void** ppout)
{
	if (riid == IID_IUnknown)
		*ppout = this;
	else if (riid == IID_IScriptModule)
		*ppout = static_cast<IScriptModule*>(this);
	else
		return E_NOINTERFACE;
	reinterpret_cast<IUnknown*>(*ppout)->AddRef();
	return S_OK;
}

ULONG __stdcall cScriptModule::AddRef(void) 
{
	return m_iRef++;
}

ULONG __stdcall cScriptModule::Release(void)
{
	// global COM objects?  First time I've ever heard of THAT
	if (m_iRef)
		m_iRef--;
	return m_iRef;
}
	
const char* __stdcall cScriptModule::GetName(void)
{
	return m_pszName;
}

const sScrClassDesc* __stdcall cScriptModule::GetFirstClass(tScrIter* pIterParam)
{
	sScrClassDesc const* pRet;
	m_iCurIndex = 0;
	pRet = GetScript(m_iCurIndex);
	return pRet;
}

const sScrClassDesc* __stdcall cScriptModule::GetNextClass(tScrIter* pIterParam) 
{
	sScrClassDesc const* pRet;
	m_iCurIndex++;
	pRet = GetScript(m_iCurIndex);

	return pRet;
}

void __stdcall cScriptModule::EndClassIter(tScrIter*)
{
	// Nothing to do here
}

const sScrClassDesc* cScriptModule::GetScript(unsigned int i)
{
	if (i < sm_ScriptsArraySize)
		return &sm_ScriptsArray[i];
	else
		return NULL;
}

cScriptModule  g_ScriptModule;
MPrintfProc    g_pfnMPrintf = NULL;

extern "C" __declspec(dllexport)
int __stdcall ScriptModuleInit (char* pszName, IScriptMan* pScriptMan, MPrintfProc pfnMPrintf, IMalloc* pMalloc, IScriptModule** pOutInterface)
{
	*pOutInterface = NULL;

	// Do we have any guarantee that these pointers will always be valid?
	// Well, I guess that scripts work at all says we can assume a little bit.
	g_pScriptManager = pScriptMan;
#ifdef DEBUG
	pMalloc->QueryInterface(IID_IDebugMalloc, reinterpret_cast<void**>(&g_pMalloc));
	if (!g_pMalloc)
		g_pMalloc = pMalloc;
#else
	g_pMalloc = pMalloc;
#endif

	g_pfnMPrintf = pfnMPrintf;

	// Just in case, let's bail if we don't have what we want.
	if (!g_pScriptManager || !g_pMalloc)
		return 0;

	//g_pScriptModule = new cScriptModule(pszName);
	g_ScriptModule.SetName(pszName);
	g_ScriptModule.QueryInterface(IID_IScriptModule, reinterpret_cast<void**>(pOutInterface));

	return 1;
}

#ifdef _HACKS
extern "C" int APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
	}
	return TRUE;
}
#endif

