/****************************************
 * Scripts by Tom N Harris (Telliamed)
 * mailto:telliamed@whoopdedo.cjb.net
 */

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef _TNHSCRIPT_H
#define _TNHSCRIPT_H

#include "lg/scrservices.h"
#include "lg/objects.h"

#include "scriptvars.h"
#include "Script.h"
#include "GenericTrap.h"
#include "GenericScale.h"
#include "GenericControl.h"
#include "GenericTrigger.h"

DECLARE_MODULE(tnhScript)


/********************
 * CommandControl
 *
 * Execute an arbitrary command.
 *
 * Inherits: GenericControl
 */
DECLARE_SCRIPT(cScr_CommandCtrl, cScrControl)
{
public:
	DECLARE_FACTORY(CommandControl,GenericControl);
	cScr_CommandCtrl()
		: cScrControl()
	{ }
	cScr_CommandCtrl(const char* pszName, int iHostObjId)
		: cScrControl(pszName,iHostObjId)
	{ }

  protected:
	virtual long OnControlString(const char*,sScrMsg*,sMultiParm*);
};


/*********************
 * TrapFrobber
 *
 * Translate a TurnOn message into FrobWorldEnd
 *
 * Inherits: GenericTrap
 * Parameters: inv
 */
DECLARE_SCRIPT(cScr_TrapFrobber, cScrTrap)
{
public:
	DECLARE_FACTORY(TrapFrobber, GenericTrap);
	cScr_TrapFrobber()
		: cScrTrap()
	{ }
	cScr_TrapFrobber(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId)
	{ }

protected:
	virtual long OnTurnOn(sScrMsg* pMsg, sMultiParm* pReply);
};


/***********************
 * TrapMoveRelative
 *
 * Move and apply velocity to an object relative to its current position
 *
 * Inherits: GenericControl
 * Links: ControlDevice(TrapObject, TargetObject)
 * Parameters: x,y,z, dx,dy,dz, h,p,b, dh,dp,db
 * Rotational velocity not supported at this time.
 */
DECLARE_SCRIPT(cScr_MoveRelativeTrap, cScrControl)
{
  public:
	DECLARE_FACTORY(TrapMoveRelative,GenericControl);
	cScr_MoveRelativeTrap()
	  : cScrControl()
	{ }
	cScr_MoveRelativeTrap(const char* pszName, int iHostObjId)
		: cScrControl(pszName,iHostObjId)
	{ }
  private:
	DECLARE_MSGHANDLER(OnSim);
	static int LinkIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

  protected:
	virtual long OnControlString(const char*,sScrMsg*,sMultiParm*);
	virtual void InitScriptData()
	{
		cScrControl::InitScriptData();
		REGISTER_MSGHANDLER("Sim", cScr_MoveRelativeTrap::OnSim);
	}
};

/**********************
 * TrapCampaignQVar
 * 
 * Set the value for a quest variable that is stored in the campaign database
 * 
 * Inherits: GenericTrap
 * Properties: Trap\QVar (see documentation of TrapSetQVar for details)
 */
DECLARE_SCRIPT(cScr_CampaignQVarTrap,cScrTrap)
{
  public:
	DECLARE_FACTORY(TrapCampaignQVar,GenericTrap);
	cScr_CampaignQVarTrap()
	  : cScrTrap()
	{ }
	cScr_CampaignQVarTrap(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId)
	{ }

  protected:
	virtual long OnSwitch(bool bTurnOn, sScrMsg* pMsg, sMultiParm* pReply);
};


/*********************
 * TrapKillEvents
 *
 * Disables user input.  Will automatically enable events after 60 seconds, or 
 * the time specified in Script\Timing.
 *
 * Inherits: GenericTrap
 * Parameters: event_mask = Binary string representing the events to disable.
 *             1000 - Allow mouse movement.
 *             0100 - Allow mouse buttons.
 *             0010 - ??? (Joystick, maybe)
 *             0001 - Allow keyboard.
 */
DECLARE_SCRIPT(cScr_KillEvents,cScrTrap)
{
public:
	DECLARE_FACTORY(TrapKillEvents,GenericTrap);
	cScr_KillEvents()
		: cScrTrap()
	{ }
	cScr_KillEvents(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId)
	{ }

private:

protected:
	virtual long OnSwitch(bool bTurnOn, sScrMsg* pMsg, sMultiParm* pReply);
};


/*************************
 * TrapCamAttach
 *
 * Attach the camera to the script object.
 *
 * Inherits: GenericTrap
 * Parameters: nolens, static, forcereturn
 */
DECLARE_SCRIPT(cScr_CamAttach,cScrTrap)
{
public:
	DECLARE_FACTORY(TrapCamAttach,GenericTrap);
	cScr_CamAttach() : cScrTrap() { }
	cScr_CamAttach(const char* pszName, int iHostObjId) : cScrTrap(pszName,iHostObjId) { }

protected:
	virtual long OnSwitch(bool bTurnOn, sScrMsg* pMsg, sMultiParm* pReply);
};


/*********************
 * TrapFreezePlayer
 *
 * Prevent the player from moving.
 *
 * Inherits: GenericScale
 * 
 */
DECLARE_SCRIPT(cScr_FreezePlayer,cScrScale)
{
public:
	DECLARE_FACTORY(TrapFreezePlayer,GenericScale);
	cScr_FreezePlayer()
		: cScrScale(),
		  SCRIPT_VAR(cScr_FreezePlayer,m_iIsFrozen)
	{ }
	cScr_FreezePlayer(const char* pszName, int iHostObjId)
		: cScrScale(pszName, iHostObjId),
		  SCRIPT_VAROBJ(cScr_FreezePlayer,m_iIsFrozen,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnBeginScript)
	{
		m_iIsFrozen.Init(0);
		return 0;
	}

	script_int m_iIsFrozen;

protected:
	virtual long OnTurnOn(sScrMsg* pMsg, sMultiParm* pReply);
	virtual long OnTurnOff(sScrMsg* pMsg, sMultiParm* pReply);
	virtual long OnScale(float fValue, sScrMsg* pMsg, sMultiParm* pReply);
	virtual void InitScriptData()
	{
		cScrScale::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_FreezePlayer::OnBeginScript);
	}
};


/***********************
 * TrapThreshold
 *
 * Relay TurnOn every n-th time. TurnOff decrements the counter unless NoOff is used.
 * The counter can be global, or tied to the message source.
 * 
 * Inherits: GenericTrap
 * Parameters: threshold, initial, floor, global
 */
DECLARE_SCRIPT(cScr_ThresholdRelay, cScrTrap)
{
  public:
	DECLARE_FACTORY(TrapThreshold,GenericTrap);
	cScr_ThresholdRelay()
		: cScrTrap(),
		  SCRIPT_VAR(cScr_ThresholdRelay,m_iGlobalCounter)
	{ }
	cScr_ThresholdRelay(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId),
		  SCRIPT_VAROBJ(cScr_ThresholdRelay,m_iGlobalCounter,iHostObjId)
	{ }

  private:
	DECLARE_MSGHANDLER(OnBeginScript);
	script_int m_iGlobalCounter;

  protected:
	virtual long OnTurnOn(sScrMsg* pMsg, sMultiParm* pReply);
	virtual long OnTurnOff(sScrMsg* pMsg, sMultiParm* pReply);
	virtual void InitScriptData()
	{
		cScrTrap::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_ThresholdRelay::OnBeginScript);
	}
};


/********************
 * TrapRandomTimer
 *
 * Relay TurnOn/TurnOff along ControlDevice links after a random delay.
 * Respects all TrapControlFlags.
 *
 * Parameters: min_time, max_time - values are inclusive.
 * Messages: TurnOn, TurnOff
 * Links: ControlDevice
 */
DECLARE_SCRIPT(cScr_RandomTimer, cScript)
{
public:
	DECLARE_FACTORY(TrapRandomTimer,CustomScript);
	cScr_RandomTimer()
		: cScript(), 
		  SCRIPT_VAR(cScr_RandomTimer,m_hTimerHandle)
	{ }
	cScr_RandomTimer(const char* pszName, int iHostObjId)
		: cScript(pszName, iHostObjId),
		  SCRIPT_VAROBJ(cScr_RandomTimer,m_hTimerHandle,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnTurnOnOff);
	DECLARE_MSGHANDLER(OnTimer);
	DECLARE_MSGHANDLER(OnBeginScript)
	{
		m_hTimerHandle.Init();
		return 0;
	}

	script_handle<tScrTimer> m_hTimerHandle;
	void RelayMessage(const char* pszMsgName);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_RandomTimer::OnBeginScript);
		REGISTER_MSGHANDLER("TurnOn",cScr_RandomTimer::OnTurnOnOff);
		REGISTER_MSGHANDLER("TurnOff",cScr_RandomTimer::OnTurnOnOff);
		REGISTER_MSGHANDLER("Timer",cScr_RandomTimer::OnTimer);
	}
};


/******************
 * TrapRepeatingRelay
 * 
 * Relay TurnOn a repeated number of times with a delay between firings.
 * TurnOff will abort unless NoOff is set.
 * Sorta like a flicker trigger that stops on its own.
 * Respects TrapControlFlag once and nooff.
 *
 * Parameters: repeat, interval
 * Messages: TurnOn, TurnOff
 * Links: ControlDevice
 */
DECLARE_SCRIPT(cScr_RepeatingRelay, cScript)
{
public:
	DECLARE_FACTORY(TrapRelayRepeat,CustomScript);
	cScr_RepeatingRelay()
		: cScript(),
		  SCRIPT_VAR(cScr_RepeatingRelay,m_iRepeatCount),
		  SCRIPT_VAR(cScr_RepeatingRelay,m_hTimerHandle)
	{ }
	cScr_RepeatingRelay(const char* pszName, int iHostObjId)
		: cScript(pszName, iHostObjId), 
		  SCRIPT_VAROBJ(cScr_RepeatingRelay,m_iRepeatCount,iHostObjId),
		  SCRIPT_VAROBJ(cScr_RepeatingRelay,m_hTimerHandle,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnBeginScript)
	{
		m_iRepeatCount.Init();
		m_hTimerHandle.Init();
		return 0;
	}
	DECLARE_MSGHANDLER(OnTurnOn);
	DECLARE_MSGHANDLER(OnTurnOff);
	DECLARE_MSGHANDLER(OnTimer);

	script_int m_iRepeatCount;
	script_handle<tScrTimer> m_hTimerHandle;

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_RepeatingRelay::OnBeginScript);
		REGISTER_MSGHANDLER("TurnOn",cScr_RepeatingRelay::OnTurnOn);
		REGISTER_MSGHANDLER("TurnOff",cScr_RepeatingRelay::OnTurnOff);
		REGISTER_MSGHANDLER("Timer",cScr_RepeatingRelay::OnTimer);
	}
};


/*****************
 * TrapRenderFlash
 *
 * Displays a render flash as described by an archetype linked from this object
 * with a RenderFlash link.
 * Also, if there is a schema linked with SoundDescription, that will be played.
 *
 * One small drawback is that this will cause the current weapon to be deselected.
 *
 * Inherits: GenericTrap
 * Messages: TurnOn
 * Links: RenderFlash(TrapObject, FlashArchetype), SoundDescription(TrapObject, SchemaArchetype)
 */
DECLARE_SCRIPT(cScr_RenderFlash,cScrTrap)
{
  public:
	DECLARE_FACTORY(TrapRenderFlash,GenericTrap);
	cScr_RenderFlash()
	  : cScrTrap()
	{ }
	cScr_RenderFlash(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId)
	{ }

  private:
	int Relink(ILinkSrv* pLS, ILinkToolsSrv* pLTS, long flavor, int source, int dest);

  protected:
	virtual long OnSwitch(bool bTurnOn, sScrMsg* pMsg, sMultiParm* pReply);
};


/*******************
 * DisarmPlayer
 *
 * Remove offensive items from the player's inventory.
 * Items are returned to the player when the trap is turned off.
 *
 * Inherits: GenericTrap
 * Parameters: nopotions=1 - don't remove potions
 */
DECLARE_SCRIPT(cScr_DisarmPlayer,cScrTrap)
{
  public:
	DECLARE_FACTORY(TrapDisarmPlayer,GenericTrap);
	cScr_DisarmPlayer()
	  : cScrTrap()
	{ }
	cScr_DisarmPlayer(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId)
	{ }

  private:
	static int DisarmIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

  protected:
	virtual long OnSwitch(bool bTurnOn, sScrMsg* pMsg, sMultiParm* pReply);
};


/********************
 * NewCrystal
 * 
 * Substitute this object for another when contained by the player.
 * Acts like the standard "Crystal" script, but lets you specify what the
 * arrow (or whatever) will be.
 * The link can be on an archetype and the script will find it, so you
 * can make custom arrows as easily as regular once. Cool, huh?
 *
 * Messages: Contained
 * Links: ScriptParams(Crystal, ArrowArchetype, "Crystal")
 */
DECLARE_SCRIPT(cScr_NewCrystal,cScript)
{
public:
	DECLARE_FACTORY(NewCrystal,CustomScript);
	cScr_NewCrystal()
	  : cScript()
	{ }
	cScr_NewCrystal(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnContained);
	long TransmogrifyCrystal(IObjectSrv* pObjSrv, int iDest);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("Contained",cScr_NewCrystal::OnContained);
	}
};


/*******************
 * DoorFrobInert
 *
 * Add and remove the FrobInert metaproperty based on the door state of this object.
 *
 * Messages: DoorOpening, DoorClose, Slain, Destroy
 * Links: ScriptParams(Door, FrobableObject, "FrobInert")
 * Parameters: breakable - once the door is slain, it will be treated as always open
 */
DECLARE_SCRIPT(cScr_DoorFrobInert, cScript)
{
public:
	DECLARE_FACTORY(DoorFrobInert,CustomScript);
	cScr_DoorFrobInert()
		: cScript(),
		  SCRIPT_VAR(cScr_DoorFrobInert,m_iDoorIsDead)
	{ }
	cScr_DoorFrobInert(const char* pszName, int iHostObjId)
		: cScript(pszName, iHostObjId),
		  SCRIPT_VAROBJ(cScr_DoorFrobInert,m_iDoorIsDead,iHostObjId)
	{ }

private:
	script_int m_iDoorIsDead;

	DECLARE_MSGHANDLER(OnSim);
	DECLARE_MSGHANDLER(OnDoorOpening);
	DECLARE_MSGHANDLER(OnDoorClose);
	DECLARE_MSGHANDLER(OnSlain);
	DECLARE_MSGHANDLER(OnDestroy);

	static int FrobInertIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("Sim",cScr_DoorFrobInert::OnSim);
		REGISTER_MSGHANDLER("DoorOpening",cScr_DoorFrobInert::OnDoorOpening);
		REGISTER_MSGHANDLER("DoorClose",cScr_DoorFrobInert::OnDoorClose);
		REGISTER_MSGHANDLER("Slain",cScr_DoorFrobInert::OnSlain);
		REGISTER_MSGHANDLER("Destroy",cScr_DoorFrobInert::OnDestroy);
	}
};


/***************************
 * UnlinkOnContained
 *
 * When the object is contained, delete all links from another object.
 * The source(s) of the links is determined from a ScriptParams link with data Unlink
 * from this object.
 *
 * Messages: Contained
 * Links: ScriptParams(ScriptObject, LinkedObject, "Unlink")
 */
DECLARE_SCRIPT(cScr_ContainedUnlink,cScript)
{
public:
	DECLARE_FACTORY(UnlinkOnContained,CustomScript);
	cScr_ContainedUnlink()
		: cScript(),
		SCRIPT_VAR(cScr_ContainedUnlink,m_iAlreadyUnlinked)
	{ }
	cScr_ContainedUnlink(const char* pszName, int iHostObjId)
		: cScript(pszName, iHostObjId),
		  SCRIPT_VAROBJ(cScr_ContainedUnlink,m_iAlreadyUnlinked,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnBeginScript)
	{
		m_iAlreadyUnlinked.Init(0);
		return 0;
	}
	DECLARE_MSGHANDLER(OnContained);

	script_int m_iAlreadyUnlinked;

	static int ScriptParamsIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);
	static int DestroyLinksIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_ContainedUnlink::OnBeginScript);
		REGISTER_MSGHANDLER("Contained",cScr_ContainedUnlink::OnContained);
	}
};


/*******************
 * TimedGrenade
 *
 * Slays the object a specified time after being thrown (not dropped). 
 * Also slays if the object is firestimmed, or if it is poked or bashed when armed.
 * The grenade can be disarmed if you frob it within a safety period,
 * but it will be slain if the timer has less than 1600ms.
 * If a RenderFlash link is present on the grenade archetype, will trigger a world flash.
 *
 * Will play Activate/ActiveLoop/Deactivate schema events.
 * A model tweq will also be activated.
 *
 * Parameters: launch_active
 * Messages: FrobWorldBegin, FrobInvEnd, Slain
 * Stimuli: FireStim, BashStim, PokeStim
 * Properties: Script\Timing, Tweq\Models
 * Links: RenderFlash(GrenadeArchetype, FlashArchetype)
 * Schema Tags: Event Activate, Event Deactivate, Event ActiveLoop, Event Death
 */
DECLARE_SCRIPT(cScr_TimedGrenade,cScript)
{
public:
	DECLARE_FACTORY(TimedGrenade,CustomScript);
	cScr_TimedGrenade()
		: cScript(),
		  SCRIPT_VAR(cScr_TimedGrenade,m_iSafety),
		  SCRIPT_VAR(cScr_TimedGrenade,m_hFuseTimer)
	{ }
	cScr_TimedGrenade(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId),
		  SCRIPT_VAROBJ(cScr_TimedGrenade,m_iSafety,iHostObjId),
		  SCRIPT_VAROBJ(cScr_TimedGrenade,m_hFuseTimer,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnBeginScript);
	DECLARE_MSGHANDLER(OnEndScript);
	DECLARE_MSGHANDLER(OnTimer);
	DECLARE_MSGHANDLER(OnSlain);
	DECLARE_MSGHANDLER(OnFrobInv);
	DECLARE_MSGHANDLER(OnFrobWorld);
	DECLARE_MSGHANDLER(OnContained);
	DECLARE_MSGHANDLER(OnCreate);
	DECLARE_MSGHANDLER(OnStimulus);
	DECLARE_MSGHANDLER(OnSchemaDone);

	script_int m_iSafety;
	script_handle<tScrTimer> m_hFuseTimer;

	void GrenadeFlash(void);
	void GrenadeActive(bool bActivate);
	void GrenadeArmed(void);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_TimedGrenade::OnBeginScript);
		REGISTER_MSGHANDLER("EndScript",cScr_TimedGrenade::OnEndScript);
		REGISTER_MSGHANDLER("Timer",cScr_TimedGrenade::OnTimer);
		REGISTER_MSGHANDLER("Slain",cScr_TimedGrenade::OnSlain);
		REGISTER_MSGHANDLER("FrobWorldBegin",cScr_TimedGrenade::OnFrobWorld);
		REGISTER_MSGHANDLER("FrobInvEnd",cScr_TimedGrenade::OnFrobInv);
		REGISTER_MSGHANDLER("Contained",cScr_TimedGrenade::OnContained);
		REGISTER_MSGHANDLER("Create",cScr_TimedGrenade::OnCreate);
		REGISTER_MSGHANDLER("SchemaDone",cScr_TimedGrenade::OnSchemaDone);
		REGISTER_MSGHANDLER("FireStimStimulus",cScr_TimedGrenade::OnStimulus);
		//REGISTER_MSGHANDLER("BashStimStimulus",cScr_TimedGrenade::OnStimulus);
		//REGISTER_MSGHANDLER("PokeStimStimulus",cScr_TimedGrenade::OnStimulus);
	}
};


/********************
 * ZeroGravRoom
 * 
 * Sets the player's gravity to zero when entered. Sets it to 100% on exit.
 * 
 * This is a copy of the script with the same name from System Shock 2.
 * 
 * Messages: PlayerRoomEnter, PlayerRoomExit
 */
DECLARE_SCRIPT(cScr_ZeroGravRoom,cScript)
{
public:
	DECLARE_FACTORY(ZeroGravRoom,CustomScript);
	cScr_ZeroGravRoom()
	  : cScript()
	{ }
	cScr_ZeroGravRoom(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnEnter);
	DECLARE_MSGHANDLER(OnExit);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("PlayerRoomEnter",cScr_ZeroGravRoom::OnEnter);
		REGISTER_MSGHANDLER("PlayerRoomExit",cScr_ZeroGravRoom::OnExit);
		//REGISTER_MSGHANDLER("CreatureRoomEnter",cScr_ZeroGravRoom::OnEnter);
		//REGISTER_MSGHANDLER("CreatureRoomExit",cScr_ZeroGravRoom::OnExit);
	}
};


/********************
 * CreatureGravRoom
 * 
 * It doesn't work.
 *
 * Messages: CreatureRoomEnter, CreatureRoomExit
DECLARE_SCRIPT(cScr_CreatGravRoom,cScript)
{
public:
	DECLARE_FACTORY(CreatureGravRoom,CustomScript);
	cScr_CreatGravRoom()
	  : cScript()
	{ }
	cScr_CreatGravRoom(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnEnter);
	DECLARE_MSGHANDLER(OnExit);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("PlayerRoomEnter",cScr_CreatGravRoom::OnEnter);
		REGISTER_MSGHANDLER("PlayerRoomExit",cScr_CreatGravRoom::OnExit);
		REGISTER_MSGHANDLER("CreatureRoomEnter",cScr_CreatGravRoom::OnEnter);
		REGISTER_MSGHANDLER("CreatureRoomExit",cScr_CreatGravRoom::OnExit);
	}
};
 */


/********************
 * TrapFadeOut
 *
 * Fade to black.
 *
 * Inherits: GenericScale
 */
DECLARE_SCRIPT(cScr_FadeOut,cScrScale)
{
public:
	DECLARE_FACTORY(TrapFadeOut,GenericScale);
	cScr_FadeOut()
		: cScrScale()
	{ }
	cScr_FadeOut(const char* pszName, int iHostObjId)
		: cScrScale(pszName, iHostObjId)
	{ }

protected:
	virtual long OnTurnOff(sScrMsg* pMsg, sMultiParm* pReply);
	virtual long OnScale(float fValue, sScrMsg* pMsg, sMultiParm* pReply);
};


/********************
 * NewKey
 *
 * Patch for StdKey to fix the PlayerToolFrob bug.
 *
 */
DECLARE_SCRIPT(cScr_NewKey,cScript)
{
public:
	DECLARE_FACTORY(NewKey,);
	cScr_NewKey() : cScript(), m_pStdKey(NULL)
	{ }
	cScr_NewKey(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId), m_pStdKey(NULL)
	{ }
	virtual ~cScr_NewKey ();

private:
	DECLARE_MSGHANDLER(OnFrobToolEnd);
	IScript* m_pStdKey;

protected:
	virtual void InitScriptData();
	virtual long __stdcall ReceiveMessage(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace);
};


/*************************
 * KnockOnDoor
 *
 * Signals AI with "knock_knock" when a closed & locked door is frobbed.
 *
 * Messages: FrobWorldEnd
 * Links: Owns(AIToSignal, DoorObject)
 */
DECLARE_SCRIPT(cScr_KnockOnDoor,cScript)
{
public:
	DECLARE_FACTORY(KnockOnDoor,CustomScript);
	cScr_KnockOnDoor()
		: cScript()
	{ }
	cScr_KnockOnDoor(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnFrob);
	static int LinkIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("FrobWorldEnd",cScr_KnockOnDoor::OnFrob);
	}
};


/**************************
 * LockHelper
 *
 * Script that sends TurnOn based on various interactions with a locked object.
 *
 * Needs PickHelper on lockpicks and KeyHelper on keys.
 * 
 * Messages: FrobWorldEnd, UseKey, Wrongkey, PickBegin, PickEnd
 * Links: ScriptParams data: FrobLock, UseKey, Wrongkey, PickBegin, PickEnd
 */
DECLARE_SCRIPT(cScr_LockHelper,cScript)
{
public:
	DECLARE_FACTORY(LockHelper,CustomScript);
	cScr_LockHelper()
		: cScript()
	{ }
	cScr_LockHelper(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnFrob);
	DECLARE_MSGHANDLER(OnMessage);
	static int LinkIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("FrobWorldEnd",cScr_LockHelper::OnFrob);
		REGISTER_MSGHANDLER("Wrongkey",cScr_LockHelper::OnMessage);
		REGISTER_MSGHANDLER("UseKey",cScr_LockHelper::OnMessage);
		REGISTER_MSGHANDLER("PickBegin",cScr_LockHelper::OnMessage);
		REGISTER_MSGHANDLER("PickEnd",cScr_LockHelper::OnMessage);
	}
};


DECLARE_SCRIPT(cScr_KeyHelper,cScript)
{
public:
	DECLARE_FACTORY(KeyHelper,CustomScript);
	cScr_KeyHelper()
		: cScript()
	{ }
	cScr_KeyHelper(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnFrob);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("FrobToolEnd",cScr_KeyHelper::OnFrob);
	}
};


DECLARE_SCRIPT(cScr_PickHelper,cScript)
{
public:
	DECLARE_FACTORY(PickHelper,CustomScript);
	cScr_PickHelper()
		: cScript()
	{ }
	cScr_PickHelper(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnFrobInvBegin);
	DECLARE_MSGHANDLER(OnFrobInvEnd);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("FrobToolBegin",cScr_PickHelper::OnFrobInvBegin);
		REGISTER_MSGHANDLER("FrobToolEnd",cScr_PickHelper::OnFrobInvEnd);
	}
};


/********************
 * NewGasLight
 *
 * Patch for GasLight that adds the ability to turn off the light by frobbing it.
 * Also supports an emitter tweq that simulates an unignited jet of gas.
 *
 */
DECLARE_SCRIPT(cScr_NewGasLight,cScript)
{
public:
	DECLARE_FACTORY(NewGasLight,);
	cScr_NewGasLight() : cScript(), m_pGasLight(NULL)
	{ }
	cScr_NewGasLight(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId), m_pGasLight(NULL)
	{ }
	virtual ~cScr_NewGasLight ();

private:
	DECLARE_MSGHANDLER(OnFrobWorldEnd);
	DECLARE_MSGHANDLER(OnTurnOff);
	DECLARE_MSGHANDLER(OnExtinguish);
	int ToggleEmitter(int iAction);

	IScript* m_pGasLight;

protected:
	virtual void InitScriptData();
	virtual long __stdcall ReceiveMessage(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace);
};


/***************************
 * TrapSiblingRelay
 *
 * Facilitates a group of relays that work together. The objects are linked in a
 * ring network using ScriptParams with data Sibling. Each object should have 
 * exactly one Sibling link from it, and exactly one to it. 
 * Normal operation is that every object will relay a TurnOn or TurnOff message 
 * along its ControlDevice links when any one of the objects receives one. All 
 * trap control flags are respected. With the parameter exclusive, the object 
 * that receives the TurnOn or TurnOff message will tell the other objects to 
 * do the opposite. The parameter may be either "on" or "off". 
 * To allow all the objects in the network to work together, the object with 
 * the lowest ID is designated as the base object. The trap control flags, 
 * timing, and locked properties of this object are used for the entire network.
 * You should replicate the properties on all the other objects anyway, since
 * IDs have a tendency to drift under certain circumstances.
 * Each object can have a number of Owns links from it to other objects, such 
 * as levers. When the object receives a message along the network, it will 
 * send an appropriate message to the owned object. Because toggling a lever 
 * causes it to send a message back to this object, the script anticipates 
 * and "swallow" these messages. If the owned objects won't cause extra 
 * messages, you can specify no_swallow. You cannot have a mix of messages that 
 * will be swallowed and some that won't. If sending a TurnOn or TurnOff message 
 * to the owned objects will cause the opposite message to be received by this
 * object, you should set invert_levers. Again, it's all or nothing. This does 
 * not include the Invert trap control flag for this object (or rather, the 
 * network the object is in). The script needs to know whether the owned objects 
 * are currently on or off. The default is to assume the levers begin in the 
 * off position. Use start_on to assume the levers are already on. Once a 
 * message is received, the state is set to that message.
 * If one of the objects in the network is destroyed, it attempts to self-heal
 * the network. Its sibling links will be connected together. If the object 
 * was the base object, it will transfer the trap control flags, timing, and 
 * locked properties to a new base object. The message NewBaseObject is used 
 * to inform the rest of the network that the base has changed.
 * 
 * And now, I'm going to bed.
 * 
 * Inherits: GenericTrap
 * Parameters: exclusive(on,off), invert_levers, no_swallow, start_on
 * Links: ScriptParams(Object, Object, "Sibling"), Owns(Relay, Lever), ControlDevice
 * Messages: TurnOn, TurnOff, NowLocked, NowUnlocked, NewBaseObject
 */
DECLARE_SCRIPT(cScr_SiblingRelay,cScrTrap)
{
public:
	DECLARE_FACTORY(TrapSiblingRelay,GenericTrap);
	cScr_SiblingRelay()
		: cScrTrap(),
		  SCRIPT_VAR(cScr_SiblingRelay,m_iBaseObjectId),
		  SCRIPT_VAR(cScr_SiblingRelay,m_giSiblingLock),
		  SCRIPT_VAR(cScr_SiblingRelay,m_iSwallowTurnOn),
		  SCRIPT_VAR(cScr_SiblingRelay,m_iSwallowTurnOff),
		  SCRIPT_VAR(cScr_SiblingRelay,m_iCurrentState)
	{ }
	cScr_SiblingRelay(const char* pszName, int iHostObjId)
		: cScrTrap(pszName, iHostObjId),
		  SCRIPT_VAROBJ(cScr_SiblingRelay,m_iBaseObjectId,iHostObjId),
		  SCRIPT_VAROBJ(cScr_SiblingRelay,m_giSiblingLock,iHostObjId),
		  SCRIPT_VAROBJ(cScr_SiblingRelay,m_iSwallowTurnOn,iHostObjId),
		  SCRIPT_VAROBJ(cScr_SiblingRelay,m_iSwallowTurnOff,iHostObjId),
		  SCRIPT_VAROBJ(cScr_SiblingRelay,m_iCurrentState,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnBeginScript);
	DECLARE_MSGHANDLER(OnDestroy);
	DECLARE_MSGHANDLER(OnNewBaseObject);
	DECLARE_MSGHANDLER(OnNowLocked);
	DECLARE_MSGHANDLER(OnNowUnlocked);

	int FindFirstObject(int iBase);
	void PropogateMessage(const char* pszMsg, int param);
	void ToggleLevers(int iState);

	static int ToggleIterFunc(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);

	script_int m_iBaseObjectId;
	// This variable is shared amongst siblings.
	// Its obj id will be set to the base object id, not this object's
	// (unless this is the base object, of course)
	script_int m_giSiblingLock;
	script_int m_iSwallowTurnOn;
	script_int m_iSwallowTurnOff;
	script_int m_iCurrentState;

protected:
	virtual long OnTurnOn(sScrMsg* pMsg, sMultiParm* pReply);
	virtual long OnTurnOff(sScrMsg*,sMultiParm*);
	virtual void InitTrapVars();
	virtual bool IsLocked(sScrMsg* pMsg);
	virtual void SetLock(bool bLock);
	virtual void InitScriptData()
	{
		cScrTrap::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript",cScr_SiblingRelay::OnBeginScript);
		REGISTER_MSGHANDLER("Destroy",cScr_SiblingRelay::OnDestroy);
		REGISTER_MSGHANDLER("NewBaseObject",cScr_SiblingRelay::OnNewBaseObject);
		REGISTER_MSGHANDLER("NowLocked",cScr_SiblingRelay::OnNowLocked);
		REGISTER_MSGHANDLER("NowUnlocked",cScr_SiblingRelay::OnNowUnlocked);
	}
};


/**************************
 * TrigContact
 *
 * Triggers when it comes in contact with another object.
 *
 * Inherits: GenericTrigger
 * Messages: PhysContactCreate, PhysContactDestroy
 * Links: ScriptParams w/ Contact
 */
DECLARE_SCRIPT(cScr_ContactTrigger,cScript), cScrTrigger
{
public:
	DECLARE_FACTORY(TrigContact,CustomScript);
	cScr_ContactTrigger()
		: cScript(), cScrTrigger()
	{ }
	cScr_ContactTrigger(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId), cScrTrigger(iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnBeginScript);
	DECLARE_MSGHANDLER(OnEndScript);
	DECLARE_MSGHANDLER(OnPhysContactCreate);
	DECLARE_MSGHANDLER(OnPhysContactDestroy);
	static int FindContactObject(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);
	static int FindContactArchetype(ILinkSrv* pLS, ILinkQuery* pLQ, IScript* pScript, void* pData);
	
protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("BeginScript", cScr_ContactTrigger::OnBeginScript);
		REGISTER_MSGHANDLER("EndScript", cScr_ContactTrigger::OnEndScript);
		REGISTER_MSGHANDLER("PhysContactCreate", cScr_ContactTrigger::OnPhysContactCreate);
		REGISTER_MSGHANDLER("PhysContactDestroy", cScr_ContactTrigger::OnPhysContactDestroy);
	}
};


/**********************
 * TrapText
 *
 * Inherits: GenericTrap
 * Properties: Book
 */
DECLARE_SCRIPT(cScr_TrapText,cScrTrap)
{
  public:
	DECLARE_FACTORY(TrapText,GenericTrap);
	cScr_TrapText()
	  : cScrTrap()
	{ }
	cScr_TrapText(const char* pszName, int iHostObjId)
		: cScrTrap(pszName,iHostObjId)
	{ }

  private:
	DECLARE_MSGHANDLER(OnFrob);

  protected:
	virtual long OnSwitch(bool bTurnOn, sScrMsg* pMsg, sMultiParm* pReply);
	virtual void InitScriptData()
	{
		cScrTrap::InitScriptData();
		REGISTER_MSGHANDLER("FrobWorldEnd",cScr_TrapText::OnFrob);
	}
};


/*****************************
 * FocusPlaque
 *
 * Display the book text on-screen when the object is hilighted.
 * Will remove the text when unfocused, unless you set NoOff.
 * 
 * Messages: WorldSelect, WorldDeSelect
 * Properties: Book
 */
DECLARE_SCRIPT(cScr_FocusPlaque,cScript)
{
public:
	DECLARE_FACTORY(FocusPlaque,CustomScript);
	cScr_FocusPlaque()
		: cScript()
	{ }
	cScr_FocusPlaque(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnFocus);
	DECLARE_MSGHANDLER(OnDeFocus);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("WorldSelect",cScr_FocusPlaque::OnFocus);
		REGISTER_MSGHANDLER("WorldDeSelect",cScr_FocusPlaque::OnDeFocus);
	}
};


/*****************************
 * ContainerPhysics
 *
 * Transfers the velocity of a container to the items it contains when 
 * the container is slain.
 * 
 * Handles death-by-collision stupidly. Reverses the impact vector, 
 * instead of reflecting it. Will scale the magnitude to 60% on a 
 * normal death, or 10% on collision.
 * Not futzing with the velocity has a tendency to throw the objects 
 * into solid space. This seems to prevent that most of the time, 
 * and looks "good enough".
 * 
 * Messages: Slain
 */
DECLARE_SCRIPT(cScr_ContainerPhysics,cScript)
{
public:
	DECLARE_FACTORY(ContainerPhysics,CustomScript);
	cScr_ContainerPhysics()
		: cScript()
	{ }
	cScr_ContainerPhysics(const char* pszName, int iHostObjId)
		: cScript(pszName,iHostObjId)
	{ }

private:
	DECLARE_MSGHANDLER(OnSlain);

protected:
	virtual void InitScriptData()
	{
		cScript::InitScriptData();
		REGISTER_MSGHANDLER("Slain",cScr_ContainerPhysics::OnSlain);
	}
};


#endif
